        PROGRAM MN_DCOD

        PARAMETER       ( MXSTNS = 300000 )
C*                                      Maximum number of stations
C*                                      in a Mesonet data file

        PARAMETER       ( NVAR = 41 )
C*                                      Number of Mesonet variables
C*                                      to retrieve for each station

	CHARACTER	mesofl*128, mesobn*128,
     +                  stid(MXSTNS)*10, prvid(MXSTNS)*10,
     +                  cstrtm*9, crpttm(MXSTNS)*9,
     +                  qcd(MXSTNS,NVAR)*1, vcname(NVAR)*7

        REAL            slat ( MXSTNS ), slon ( MXSTNS ),
     +                  selv ( MXSTNS ), obs ( MXSTNS, NVAR )

        INTEGER         istnm ( MXSTNS ), isrcn ( MXSTNS ),
     +                  iqca ( MXSTNS, NVAR ), iqcr ( MXSTNS, NVAR )
C*
C*      The following variable code names define, in accordance
C*      with the MADIS table files "static/sfctbl.txt" and
C*      "static/sfcvcn.txt", the variables that will be read
C*      from the Mesonet data file.
C*
        DATA            vcname
     +                  / 'TD     ', 'ALTSE  ', 'P      ', 'T      ',
     +                    'DD     ', 'FF     ', 'SOILM  ', 'SOILT  ',
     +                    'PCP1H  ', 'PCP24H ', 'PCPUTCM', 'PCPRATE',
     +                    'DDGUST ', 'FFGUST ',
     +                    'RDT1', 'RDT2', 'RDT3', 'RDT4' ,
     +                    'RDLFT1', 'RDLFT2', 'RDLFT3', 'RDLFT4' ,
     +                    'RDLIP1', 'RDLIP2', 'RDLIP3', 'RDLIP4' ,
     +                    'RDLDP1', 'RDLDP2', 'RDLDP3', 'RDLDP4' ,
     +                    'RDSTA1', 'RDSTA2', 'RDSTA3', 'RDSTA4' ,
     +                    'FSRD15M', 'FSRD1H ', 'FSRD24H',
     +                    'DSRD15M', 'DSRD1H ', 'DSRD24H',
     +                    'VIS' /
C*-----------------------------------------------------------------------
	iret = 0
C*
        mesofl = 'mesonet/20210825_2000.2133'
        mesobn = '20210825_2000.2133'

        OPEN ( 51, FILE = 'run.out' )

C*	The MADIS (Meteorological Assimilation Data Ingest System)
C*	software from FSL will be used to read the Mesonet data file.
C*	Initialize this software.
C
	CALL MINIT  ( 'SFC', 'FSL', .false., ierfin )
	IF  ( ierfin .ne. 0 )  THEN
            print *, 'bad return from MINIT with ierfin = ', ierfin
            STOP
	END IF
	CALL MSETSFCPVDR  ( 'ALL-SFC', .false., ierssp1 )
	IF  ( ierssp1 .ne. 0 )  THEN
            print *, 'bad return from MSETSFCPVDR with ierssp1 = ',
     +               ierssp1
	    STOP
	END IF
	CALL MSETSFCPVDR  ( 'ALL-MESO', .true., ierssp2 )
	IF  ( ierssp2 .ne. 0 )  THEN
            print *, 'bad return from MSETSFCPVDR with ierssp2 = ',
     +               ierssp2
	    STOP
	END IF
C        
C*      Using the Mesonet data file basename,
C*      compute the MADIS date-time.
C
        READ  ( UNIT = mesobn, FMT = '( I4.4, 2I2.2, 1X, 2I2.2 )',
     +          IOSTAT = ier )
     +      ifyear, ifmnth, ifdays, ifhour, ifminu
        CALL MCHRTIM  ( ifyear, ifmnth, ifdays, ifhour, ifminu,
     +                  cstrtm, ierfct )
        IF  ( ierfct .ne. 0 )  THEN
            print *, 'bad return from MCHRTIM with ierfct = ', ierfct
            STOP
        END IF
C
C*      Retrieve all of the stations for which there is data in the
C*      Mesonet data file.
C
        CALL MSFCSTA  ( cstrtm, nstns, stid, istnm, slat, slon, selv,
     +                  crpttm, prvid, ierfst, isrcn )
        IF  ( ierfst .ne. 0 )  THEN
            IF  ( ierfst .eq. 1006 )  THEN
                print *, 'contained no reports'
            ELSE
                print *, 'bad return from MSFCSTA with ierfst = ',
     +                   ierfst
            END IF
            STOP
        END IF
        write (51,FMT='(A,I7)') 'number of retrieved stations =', nstns
        write (51,FMT='(2A)') 
     +   '            STID    PROVIDER     RPTTIM',
     +   '        LAT     LONG      ELEV'
        DO ii = 1, nstns
            write (51,FMT='(I7,3(2X,A),3F10.2)')
     +           ii, stid(ii), prvid(ii), crpttm(ii),
     +           slat(ii), slon(ii), selv(ii)
        END DO
C
C*      Now, retrieve the data.
C
        DO ii = 1, NVAR
            CALL MGETSFC  ( cstrtm, vcname ( ii ), itmp, nnmsg,
     +                      obs  ( 1, ii ), qcd  ( 1, ii ),
     +                      iqca ( 1, ii ), iqcr ( 1, ii ), ierfgs )
            IF  ( ( ierfgs .ne. 0 ) .and.
     +            ( ierfgs .ne. 1006 ) .and.
     +            ( ierfgs .ne. 1005 ) ) THEN
                print *, 'bad return from MGETSFC with ierfgs = ',
     +                   ierfgs
                STOP
            END IF
        END DO
        DO ii = 1, nstns
            write (51,FMT='(/,A,I7)') 'data for station #',ii
            write (51,FMT='(A,8F10.2)') 'obs = ',
     +              (obs(ii,jj),jj=1,6), obs(ii,9), obs(ii,10)
            write (51,FMT='(A,8(2X,A))') 'qcd = ',
     +              (qcd(ii,jj),jj=1,6), qcd(ii,9), qcd(ii,10)
            write (51,FMT='(A,8I7)') 'iqca = ',
     +              (iqca(ii,jj),jj=1,6), iqca(ii,9), iqca(ii,10)
            write (51,FMT='(A,8I7)') 'iqcr = ',
     +              (iqcr(ii,jj),jj=1,6), iqcr(ii,9), iqcr(ii,10)
        END DO
C
        STOP
        END
